/*-
 * Copyright (c) 1993, Trusted Information Systems, Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Consultants incorporating this software in deliverables must reproduce
 *    this list of conditions and the following disclaimer within any
 *    end-user documentation.
 * 4. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Trusted Information
 *		Systems, Incorporated
 * 5. The name of Trusted Information Systems, Incorporated may not
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Trusted Information Systems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Trusted Information Systems BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 *	Author: Marcus J. Ranum, Trusted Information Systems, Inc.
 */


#include "rand.h"
#if NRAND > 0

/*
	Character driver for /dev/random. Marcus J. Ranum, 1993
	Trusted Information Systems, Inc.

	DEBUG version of driver. Copies values to user space based
	on minor number. This is so we can try to get regressions to
	see how good the seed really is.

*/

#include        <sys/param.h>
#include        <sys/systm.h>
#include        <sys/mbuf.h>
#include        <sys/file.h>
#include        <sys/socket.h>
#include        <sys/errno.h>
#include        <sys/ioctl.h>
#include        <sys/user.h>
#include        <sys/proc.h>
#include        <sys/uio.h>
 

#include "global.h"
#include "md5.h"

/* system specific library routine for seeding engine */
extern	void	rand_sys_specific();


/* state of MD5 result saved across calls for feedback */
static	unsigned char	saved_state[13];

#ifdef	RANDDEBUG
#define	RANDPRINT(s,b) rand_print_state(s,b)
#else
#define	RANDPRINT(s,b)
#endif


static	void
xor_states(s1,s2)
unsigned char	s1[];
unsigned char	s2[];
{
	s1[0] ^= s2[0];
	s1[1] ^= s2[1];
	s1[2] ^= s2[2];
	s1[3] ^= s2[3];
	s1[4] ^= s2[4];
	s1[5] ^= s2[5];
	s1[6] ^= s2[6];
	s1[7] ^= s2[7];
	s1[8] ^= s2[8];
	s1[9] ^= s2[9];
	s1[10] ^= s2[10];
	s1[11] ^= s2[11];
	s1[12] ^= s2[12];
}


static	void
incr_state(s1)
unsigned char	s1[];
{
	s1[0]++;
	s1[1]++;
	s1[2]++;
	s1[3]++;
	s1[4]++;
	s1[5]++;
	s1[6]++;
	s1[7]++;
	s1[8]++;
	s1[9]++;
	s1[10]++;
	s1[11]++;
	s1[12]++;
}



randread (dev, uio)
dev_t		dev;
struct uio     *uio;
{
	MD5_CTX			c;
	int			len;
	int			error = 0;
	unsigned char		n_state[13];

	/* snapshot seeding stuff from state of system and MD5 it */
	rand_sys_specific(n_state);
	RANDPRINT("  New State",n_state);

	/* fold in saved state from prior calls */
	xor_states(n_state,saved_state);
	RANDPRINT("Xored State",n_state);

	len = MIN(uio->uio_resid,sizeof(n_state));
	MD5Init(&c);
	MD5Update(&c,n_state,sizeof(n_state));
	MD5Final(n_state,&c);

	error = uiomove((caddr_t)n_state,len,UIO_READ,uio);
	RANDPRINT("Return data",n_state);
	incr_state(n_state);

	/* save new state as old state */
	bcopy(n_state,saved_state,sizeof(n_state));
	RANDPRINT(" Save State",saved_state);

	return(error);
}



randopen(dev,flag)
dev_t		dev;
{
	return(0);
}


#ifdef	RANDDEBUG
rand_print_state(s,buf)
char			*s;
unsigned	char	buf[];
{
	int		x;
	static char	*xab = "0123456789ABCDEF";  
 
        printf("%s: ",s);
	for(x = 0; x < 13; x++) 
		printf("%c%c",xab[buf[x] >> 4],xab[buf[x] & 0x0F]);
	printf("\n");
}
#endif
#endif	NRAND
