/*-
 * Copyright (c) 1993, Trusted Information Systems, Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Consultants incorporating this software in deliverables must reproduce
 *    this list of conditions and the following disclaimer within any
 *    end-user documentation.
 * 4. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Trusted Information
 *		Systems, Incorporated
 * 5. The name of Trusted Information Systems, Incorporated may not
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Trusted Information Systems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Trusted Information Systems BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 *	Author: Marcus J. Ranum, Trusted Information Systems, Inc.
 */



#include        <sys/param.h> 
#include        <sys/systm.h> 
#include        <sys/mbuf.h> 
#include        <sys/file.h>
#include        <sys/socket.h>
#include        <sys/errno.h> 
#include        <sys/ioctl.h> 
#include        <sys/user.h>
#include        <sys/proc.h>
#include        <sys/uio.h> 
#include        <sys/buf.h> 
  


#include "global.h"
#include "md5.h"

/*
	/dev/random system specific routine for gathering seeds.

	This routine is expected to gather system state and time
specific values from within the kernel, and return an MD5 hash
value from them.
*/


/* stuff process table values */
#define	CRUNCH_PROCTABLE

/* stuff buffer cache lru chains */
#define	CRUNCH_BUFCACHE



void
rand_sys_specific(digest)
unsigned char	digest[];
{
	MD5_CTX			c;
#ifdef	CRUNCH_PROCTABLE
	register struct proc	*p;
#endif
#ifdef	CRUNCH_BUFCACHE
	register struct buf	*b;
	int			x;
#endif
#ifdef	CRUNCH_BOOTTIME
	extern	time_t		boottime;
#endif

	MD5Init(&c);

#ifdef	CRUNCH_BOOTTIME
	MD5Update(&c,(caddr_t)&boottime,sizeof(bootime));
#endif

#ifdef	CRUNCH_PROCTABLE
	for(p = allproc; p != (struct proc *)0; p = p->p_nxt) {
		MD5Update(&c,(caddr_t)&p->p_pid,sizeof(short));
		MD5Update(&c,(caddr_t)&p->p_wchan,sizeof(caddr_t));
		MD5Update(&c,(caddr_t)&p->p_cpticks,sizeof(short));
		MD5Update(&c,(caddr_t)&p->p_stack,sizeof(caddr_t));
		MD5Update(&c,(caddr_t)&p->p_cpu,sizeof(char));
		MD5Update(&c,(caddr_t)&p->p_flag,sizeof(int));
	}
#endif


#ifdef	CRUNCH_BUFCACHE
	for(x = 0; x < BUFHSZ; x++) {
		b = bufhash[x].b_forw;
		for(b= b->b_forw; b != bufhash[x].b_forw; b = b->b_forw) {
			MD5Update(&c,(caddr_t)&b->b_flags,sizeof(long));
			MD5Update(&c,(caddr_t)&b->b_blkno,sizeof(daddr_t));
			MD5Update(&c,(caddr_t)&b->b_vp,sizeof(caddr_t));
		}
	}
#endif

	MD5Final(digest,&c);
}
